<?php

namespace Drupal\liveblog\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\liveblog\Entity\LiveblogPost;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\Component\Utility\Html;
use Drupal\liveblog\LiveblogRendererInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller providing the resource for the form.
 */
class LiveblogController extends ControllerBase {

  /**
   * Renders content in a special way.
   *
   * @var \Drupal\liveblog\LiveblogRendererInterface
   */
  protected $liveblogRenderer;

  /**
   * Construct a LiveblogController object.
   *
   * @param \Drupal\liveblog\LiveblogRendererInterface $liveblog_renderer
   *   Renders content in a special way.
   */
  public function __construct(LiveblogRendererInterface $liveblog_renderer) {
    $this->liveblogRenderer = $liveblog_renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('liveblog.renderer')
    );
  }

  /**
   * Returns a liveblog post form wrapped in a json response.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response containing the liveblog post form.
   */
  public function getFormAsJson(LiveblogPost $liveblog_post) {
    Html::setIsAjax(TRUE);

    $form_object = $this->entityTypeManager()
      ->getFormObject('liveblog_post', 'edit')
      ->setEntity($liveblog_post);
    $content = $this->formBuilder()->getForm($form_object);

    return new JsonResponse($this->liveblogRenderer->render($content));
  }

}
